;;; DO NOT MODIFY THIS FILE
(if (featurep 'erc-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "erc/_pkg.el")

(package-provide 'erc :version 0.05 :author-version "Version 4.0 Revision: 1.608" :type 'regular)

;;;***

;;;### (autoloads nil "erc-autoaway" "erc/erc-autoaway.el")
(autoload 'erc-autoaway-mode "erc-autoaway")

;;;***

;;;### (autoloads nil "erc-autojoin" "erc/erc-autojoin.el")
(autoload 'erc-autojoin-mode "erc-autojoin" nil t)

;;;***

;;;### (autoloads nil "erc-button" "erc/erc-button.el")
(autoload 'erc-button-mode "erc-button" nil t)

;;;***

;;;### (autoloads (erc-complete) "erc-complete" "erc/erc-complete.el")

(autoload 'erc-complete "erc-complete" "\
Complete nick at point.
See `erc-try-complete-nick' for more technical info.
This function is obsolete, use `erc-pcomplete' instead." t nil)

;;;***

;;;### (autoloads (erc-ctcp-query-DCC pcomplete/erc-mode/DCC erc-cmd-DCC) "erc-dcc" "erc/erc-dcc.el")

(autoload 'erc-cmd-DCC "erc-dcc" "\
Parser for /dcc command.
This figures out the dcc subcommand and calls the appropriate routine to
handle it.  The function dispatched should be named \"erc-dcc-do-FOO-command\",
where FOO is one of CLOSE, GET, SEND, LIST, CHAT, etc." nil nil)

(autoload 'pcomplete/erc-mode/DCC "erc-dcc" "\
Provides completion for the /DCC command." nil nil)

(defvar erc-ctcp-query-DCC-hook '(erc-ctcp-query-DCC) "\
Hook variable for CTCP DCC queries")

(autoload 'erc-ctcp-query-DCC "erc-dcc" "\
The function called when a CTCP DCC request is detected by the client.
It examines the DCC subcommand, and calls the appropriate routine for
that subcommand." nil nil)

;;;***

;;;### (autoloads (erc-ezb-initialize erc-ezb-select-session erc-ezb-select erc-ezb-add-session erc-ezb-end-of-session-list erc-ezb-init-session-list erc-ezb-identify erc-ezb-notice-autodetect erc-ezb-lookup-action erc-ezb-get-login erc-cmd-ezb) "erc-ezbounce" "erc/erc-ezbounce.el")

(autoload 'erc-cmd-ezb "erc-ezbounce" "\
Send EZB commands to the EZBouncer verbatim." nil nil)

(autoload 'erc-ezb-get-login "erc-ezbounce" "\
Return an appropriate EZBounce login for SERVER and PORT.
Look up entries in `erc-ezb-login-alist'. If the username or password
in the alist is `nil', prompt for the apropriate values." nil nil)

(autoload 'erc-ezb-lookup-action "erc-ezbounce" nil nil nil)

(autoload 'erc-ezb-notice-autodetect "erc-ezbounce" "\
React on an EZBounce NOTICE request." nil nil)

(autoload 'erc-ezb-identify "erc-ezbounce" "\
Identify to the EZBouncer server." nil nil)

(autoload 'erc-ezb-init-session-list "erc-ezbounce" "\
Reset the EZBounce session list to NIL." nil nil)

(autoload 'erc-ezb-end-of-session-list "erc-ezbounce" "\
Indicate the end of the EZBounce session listing." nil nil)

(autoload 'erc-ezb-add-session "erc-ezbounce" "\
Add an EZBounce session to the session list." nil nil)

(autoload 'erc-ezb-select "erc-ezbounce" "\
Select an IRC server to use by EZBounce, in ERC style." nil nil)

(autoload 'erc-ezb-select-session "erc-ezbounce" "\
Select a detached EZBounce session." nil nil)

(autoload 'erc-ezb-initialize "erc-ezbounce" "\
Add EZBouncer convenience functions to ERC." nil nil)

;;;***

;;;### (autoloads (erc-fill) "erc-fill" "erc/erc-fill.el")
(autoload 'erc-fill-mode "erc-fill" nil t)

(autoload 'erc-fill "erc-fill" "\
Fill a region using the function referenced in `erc-fill-function'.
You can put this on `erc-insert-modify-hook' and/or `erc-send-modify-hook'." nil nil)

;;;***

;;;### (autoloads (erc-create-imenu-index) "erc-imenu" "erc/erc-imenu.el")

(autoload 'erc-create-imenu-index "erc-imenu" nil nil nil)

;;;***

;;;### (autoloads (erc-chanlist erc-cmd-LIST) "erc-list" "erc/erc-list.el")

(autoload 'erc-cmd-LIST "erc-list" "\
Displays a buffer containing a list of channels on the current server." t nil)

(autoload 'erc-chanlist "erc-list" "\
Show a channel listing of the current server in a special mode.
Please note that this function only works with IRC servers which conform
to RFC and send the LIST header (#321) at start of list transmission." t nil)

;;;***

;;;### (autoloads (erc-save-buffer-in-logs erc-logging-enabled) "erc-log" "erc/erc-log.el")
(autoload 'erc-log-mode "erc-log" nil t)

(autoload 'erc-logging-enabled "erc-log" "\
Return non-nil if logging is enabled for BUFFER.
If BUFFER is nil, the value of `current-buffer' is used.
Logging is enabled if `erc-log-channels-directory' is non-nil, the directory
is writeable (it will be created as necessary) and
`erc-enable-logging' returns a non-nil value." nil nil)

(autoload 'erc-save-buffer-in-logs "erc-log" "\
Append BUFFER contents to the log file, if logging is enabled.
If BUFFER is not provided, current buffer is used.
Logging is enabled if `erc-logging-enabbled' returns non-nil.

This is normally done on exit, to save the unsaved portion of the
buffer, since only the text that runs off the buffer limit is logged
automatically.

You can save every individual message by putting this function on
`erc-insert-post-hook'." t nil)

;;;***

;;;### (autoloads (erc-delete-dangerous-host erc-add-dangerous-host erc-delete-keyword erc-add-keyword erc-delete-fool erc-add-fool erc-delete-pal erc-add-pal) "erc-match" "erc/erc-match.el")
(autoload 'erc-match-mode "erc-match")

(autoload 'erc-add-pal "erc-match" "\
Add pal interactively to `erc-pals'." t nil)

(autoload 'erc-delete-pal "erc-match" "\
Delete pal interactively to `erc-pals'." t nil)

(autoload 'erc-add-fool "erc-match" "\
Add fool interactively to `erc-fools'." t nil)

(autoload 'erc-delete-fool "erc-match" "\
Delete fool interactively to `erc-fools'." t nil)

(autoload 'erc-add-keyword "erc-match" "\
Add keyword interactively to `erc-keywords'." t nil)

(autoload 'erc-delete-keyword "erc-match" "\
Delete keyword interactively to `erc-keywords'." t nil)

(autoload 'erc-add-dangerous-host "erc-match" "\
Add dangerous-host interactively to `erc-dangerous-hosts'." t nil)

(autoload 'erc-delete-dangerous-host "erc-match" "\
Delete dangerous-host interactively to `erc-dangerous-hosts'." t nil)

;;;***

;;;### (autoloads (erc-server-select) "erc-nets" "erc/erc-nets.el")

(autoload 'erc-server-select "erc-nets" "\
Interactively select a server using the `erc-networks-alist' and
`erc-server-alist' to connect to." t nil)

;;;***

;;;### (autoloads (erc-cmd-WHOLEFT) "erc-netsplit" "erc/erc-netsplit.el")
(autoload 'erc-netsplit-mode "erc-netsplit")

(autoload 'erc-cmd-WHOLEFT "erc-netsplit" "\
Show who's gone." nil nil)

;;;***

;;;### (autoloads nil "erc-nickserv" "erc/erc-nickserv.el")
(autoload 'erc-services-mode "erc-nickserv" nil t)

;;;***

;;;### (autoloads (pcomplete/erc-mode/NOTIFY erc-cmd-NOTIFY) "erc-notify" "erc/erc-notify.el")
(autoload 'erc-notify-mode "erc-notify" nil t)

(autoload 'erc-cmd-NOTIFY "erc-notify" "\
Change `erc-notify-list' or list current notify-list members online.
Without args, list the current list of notificated people online,
with args, toggle notify status of people." nil nil)

(autoload 'pcomplete/erc-mode/NOTIFY "erc-notify" nil nil nil)

;;;***

;;;### (autoloads nil "erc-page" "erc/erc-page.el")
(autoload 'erc-page-mode "erc-page")

;;;***

;;;### (autoloads nil "erc-pcomplete" "erc/erc-pcomplete.el")
(autoload 'erc-completion-mode "erc-pcomplete" nil t)

;;;***

;;;### (autoloads nil "erc-replace" "erc/erc-replace.el")
(autoload 'erc-replace-mode "erc-replace")

;;;***

;;;### (autoloads nil "erc-ring" "erc/erc-ring.el")
(autoload 'erc-ring-mode "erc-ring" nil t)

;;;***

;;;### (autoloads nil "erc-sound" "erc/erc-sound.el")
(autoload 'erc-sound-mode "erc-sound")

;;;***

;;;### (autoloads nil "erc-stamp" "erc/erc-stamp.el")
(autoload 'erc-timestamp-mode "erc-stamp" nil t)

;;;***

;;;### (autoloads nil "erc-track" "erc/erc-track.el")
(autoload 'erc-track-mode "erc-track" nil t)
(autoload 'erc-track-when-inactive-mode "erc-track" nil t)

;;;***

;;;### (autoloads (erc-truncate-buffer erc-truncate-buffer-to-size) "erc-truncate" "erc/erc-truncate.el")
(autoload 'erc-truncate-mode "erc-truncate" nil t)

(autoload 'erc-truncate-buffer-to-size "erc-truncate" "\
Truncates the buffer to the size SIZE.
If BUFFER is not provided, the current buffer is assumed.  The deleted
region is logged if `erc-logging-enabbled' returns non-nil." nil nil)

(autoload 'erc-truncate-buffer "erc-truncate" "\
Truncates the current buffer to `erc-max-buffer-size'.
Meant to be used in hooks, like `erc-insert-post-hook'." t nil)

;;;***

;;;### (autoloads (erc-xdcc-add-file) "erc-xdcc" "erc/erc-xdcc.el")

(autoload 'erc-xdcc-add-file "erc-xdcc" "\
Add a file to `erc-xdcc-files'." t nil)

;;;***

;;;### (autoloads (erc-select) "erc" "erc/erc.el")

(autoload 'erc-select "erc" "\
Interactively select connection parameters and run ERC.
Optional argument SERVER uses server as default for the input query.
Optional argument PORT uses passed port as default for the input query.
Optional argument NICK uses the passed nick as default for the input query." t nil)

;;;***

(provide 'erc-autoloads)
