;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'erc-match '("erc-match"))
(custom-add-loads 'erc-log '("erc-log"))
(custom-add-loads 'erc-dcc '("erc-dcc" "erc-xdcc"))
(custom-add-loads 'erc-page '("erc-page"))
(custom-add-loads 'erc '("erc-autoaway" "erc-autojoin" "erc-bbdb" "erc-button" "erc-chess" "erc-complete" "erc-dcc" "erc-fill" "erc-ibuffer" "erc-list" "erc-log" "erc-nets" "erc-netsplit" "erc-nickserv" "erc-notify" "erc-pcomplete" "erc-replace" "erc-sound" "erc-speak" "erc-stamp" "erc-truncate" "erc"))
(custom-add-loads 'erc-stamp '("erc-stamp"))
(custom-add-loads 'erc-sound '("erc-sound"))
(custom-add-loads 'erc-server-hooks '("erc"))
(custom-add-loads 'erc-speak '("erc-speak"))
(custom-add-loads 'erc-hooks '("erc-members" "erc-netsplit" "erc"))
(custom-add-loads 'erc-autoaway '("erc-autoaway"))
(custom-add-loads 'erc-netsplit '("erc-netsplit"))
(custom-add-loads 'erc-services '("erc-nickserv"))
(custom-add-loads 'erc-autojoin '("erc-autojoin"))
(custom-add-loads 'erc-chess '("erc-chess"))
(custom-add-loads 'erc-notify '("erc-notify"))
(custom-add-loads 'erc-ibuffer '("erc-ibuffer"))
(custom-add-loads 'erc-fill '("erc-fill"))
(custom-add-loads 'processes '("erc"))
(custom-add-loads 'erc-faces '("erc-button" "erc-match" "erc-stamp" "erc"))
(custom-add-loads 'erc-button '("erc-button"))
(custom-add-loads 'erc-track '("erc-track"))
(custom-add-loads 'erc-bbdb '("erc-bbdb"))

;;; custom-load.el ends here
